@@ -28,3 +28,24 @@ my $json
 $parser->parse($json);
 $parser->parse_complete();
 print $generator->get_buf . "\n";
+
+=head1 NAME
+
+json_reformat.pl - Reformat and reindent JSON
+
+=head1 DESCRIPTION
+
+This example program uses both the parsing and generating parts of
+L<JSON::YAJL> to reformat and reindent JSON.
+
+=head1 AUTHOR
+
+Leon Brocard <acme@astray.com>
+
+=head1 LICENSE
+
+This module is free software; you can redistribute it or modify it under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<JSON::YAJL>
